@@ -1284,7 +1284,7 @@ static int setup_chmap(void)
 		}
 		if (i >= hw_chmap->channels) {
 			char buf[256];
-			error(_("Channel %d doesn't match with hw_parmas"), ch);
+			error(_("Channel %d doesn't match with hw_params"), ch);
 			snd_pcm_chmap_print(hw_chmap, sizeof(buf), buf);
 			fprintf(stderr, "hardware chmap = %s\n", buf);
 			free(hw_chmap);
@@ -1765,12 +1765,12 @@ static void print_vu_meter(signed int *perc, signed int *maxperc)
 }
 
 /* peak handler */
-static void compute_max_peak(u_char *data, size_t count)
+static void compute_max_peak(u_char *data, size_t samples)
 {
 	signed int val, max, perc[2], max_peak[2];
-	static	int	run = 0;
-	size_t ocount = count;
-	int	format_little_endian = snd_pcm_format_little_endian(hwparams.format);	
+	static int run = 0;
+	size_t osamples = samples;
+	int format_little_endian = snd_pcm_format_little_endian(hwparams.format);
 	int ichans, c;
 
 	if (vumeter == VUMETER_STEREO)
@@ -1784,7 +1784,7 @@ static void compute_max_peak(u_char *data, size_t count)
 		signed char *valp = (signed char *)data;
 		signed char mask = snd_pcm_format_silence(hwparams.format);
 		c = 0;
-		while (count-- > 0) {
+		while (samples-- > 0) {
 			val = *valp++ ^ mask;
 			val = abs(val);
 			if (max_peak[c] < val)
@@ -1799,9 +1799,8 @@ static void compute_max_peak(u_char *data, size_t count)
 		signed short mask = snd_pcm_format_silence_16(hwparams.format);
 		signed short sval;
 
-		count /= 2;
 		c = 0;
-		while (count-- > 0) {
+		while (samples-- > 0) {
 			if (format_little_endian)
 				sval = le16toh(*valp);
 			else
@@ -1819,9 +1818,8 @@ static void compute_max_peak(u_char *data, size_t count)
 		unsigned char *valp = data;
 		signed int mask = snd_pcm_format_silence_32(hwparams.format);
 
-		count /= 3;
 		c = 0;
-		while (count-- > 0) {
+		while (samples-- > 0) {
 			if (format_little_endian) {
 				val = valp[0] | (valp[1]<<8) | (valp[2]<<16);
 			} else {
@@ -1844,9 +1842,8 @@ static void compute_max_peak(u_char *data, size_t count)
 		signed int *valp = (signed int *)data;
 		signed int mask = snd_pcm_format_silence_32(hwparams.format);
 
-		count /= 4;
 		c = 0;
-		while (count-- > 0) {
+		while (samples-- > 0) {
 			if (format_little_endian)
 				val = le32toh(*valp);
 			else
@@ -1895,8 +1892,8 @@ static void compute_max_peak(u_char *data, size_t count)
 		print_vu_meter(perc, maxperc);
 		fflush(stderr);
 	}
-	else if(verbose==3) {
-		fprintf(stderr, _("Max peak (%li samples): 0x%08x "), (long)ocount, max_peak[0]);
+	else if (verbose==3) {
+		fprintf(stderr, _("Max peak (%li samples): 0x%08x "), (long)osamples, max_peak[0]);
 		for (val = 0; val < 20; val++)
 			if (val <= perc[0] / 5)
 				putc('#', stderr);
@@ -2135,7 +2132,9 @@ static ssize_t pcm_read(u_char *data, size_t rcount)
 		count = chunk_size;
 	}
 
-	while (count > 0 && !in_aborting) {
+	while (count > 0) {
+		if (in_aborting)
+			goto abort;
 		if (test_position)
 			do_test_position();
 		check_stdin();
@@ -2161,6 +2160,7 @@ static ssize_t pcm_read(u_char *data, size_t rcount)
 			data += r * bits_per_frame / 8;
 		}
 	}
+abort:
 	return rcount;
 }
 
@@ -2174,7 +2174,9 @@ static ssize_t pcm_readv(u_char **data, unsigned int channels, size_t rcount)
 		count = chunk_size;
 	}
 
-	while (count > 0 && !in_aborting) {
+	while (count > 0) {
+		if (in_aborting)
+			goto abort;
 		unsigned int channel;
 		void *bufs[channels];
 		size_t offset = result;
@@ -2206,6 +2208,7 @@ static ssize_t pcm_readv(u_char **data, unsigned int channels, size_t rcount)
 			count -= r;
 		}
 	}
+abort:
 	return rcount;
 }
 
@@ -3037,6 +3040,7 @@ static int new_capture_file(char *name, char *namebuf, size_t namelen,
 
 	/* get a copy of the original filename */
 	strncpy(buf, name, sizeof(buf));
+	buf[sizeof(buf)-1] = '\0';
 
 	/* separate extension from filename */
 	s = buf + strlen(buf);
@@ -3199,11 +3203,12 @@ static void capture(char *orig_name)
 			size_t c = (rest <= (off64_t)chunk_bytes) ?
 				(size_t)rest : chunk_bytes;
 			size_t f = c * 8 / bits_per_frame;
-			if (pcm_read(audiobuf, f) != f) {
+			size_t read = pcm_read(audiobuf, f);
+			size_t save;
+			if (read != f)
 				in_aborting = 1;
-				break;
-			}
-			if (xwrite(fd, audiobuf, c) != c) {
+			save = read * bits_per_frame / 8;
+			if (xwrite(fd, audiobuf, save) != save) {
 				perror(name);
 				in_aborting = 1;
 				break;
