@@ -106,14 +106,14 @@ static int load(const char *source_file, void **dst, size_t *dst_size)
 		}
 		buf = buf2;
 	}
-	if (fd != fileno(stdin))
-		close(fd);
 	if (r < 0) {
 		fprintf(stderr, _("Read error: %s\n"), strerror(-errno));
-		free(buf);
 		goto _err;
 	}
 
+	if (fd != fileno(stdin))
+		close(fd);
+
 	*dst = buf;
 	*dst_size = pos;
 	return 0;
@@ -183,7 +183,9 @@ static int save(const char *output_file, void *buf, size_t size)
 	if (r < 0) {
 		fprintf(stderr, _("Write error: %s\n"), strerror(-errno));
 		if (fd != fileno(stdout)) {
-			remove(fname);
+			if (fname && remove(fname))
+				fprintf(stderr, _("Unable to remove file %s: %s\n"),
+						fname, strerror(-errno));
 			close(fd);
 		}
 		return 1;
